@@ -21,20 +21,24 @@ try:
     import pybedtools
 except ImportError:
     pybedtools = None
+import toolz as tz
 
 from bcbio import bam, broad, utils
+from bcbio.bam import ref
 from bcbio.log import logger
 from bcbio.distributed import multi, prun
 from bcbio.distributed.split import parallel_split_combine
 from bcbio.distributed.transaction import file_transaction
-from bcbio.pipeline import shared
+from bcbio.pipeline import config_utils, shared
+from bcbio.provenance import do
+from bcbio.variation import bedutils
 from bcbio.variation import multi as vmulti
 
 def parallel_callable_loci(in_bam, ref_file, config):
     num_cores = config["algorithm"].get("num_cores", 1)
     config = copy.deepcopy(config)
-    config["algorithm"]["memory_adjust"] = {"direction": "decrease", "magnitude": 2}
-    data = {"work_bam": in_bam, "sam_ref": ref_file, "config": config}
+    data = {"work_bam": in_bam, "config": config,
+            "reference": {"fasta": {"base": ref_file}}}
     parallel = {"type": "local", "cores": num_cores, "module": "bcbio.distributed"}
     items = [[data]]
     with prun.start(parallel, items, config, multiplier=int(num_cores)) as runner:
@@ -59,48 +63,59 @@ def calc_callable_loci(data, region=None, out_file=None):
              "min": utils.get_in(data, ("config", "algorithm", "coverage_depth_min"), 4)}
     if not utils.file_exists(out_file):
         with file_transaction(out_file) as tx_out_file:
-            bam.index(data["work_bam"], data["config"])
-            with contextlib.closing(pysam.Samfile(data["work_bam"], "rb")) as in_bam:
-                with open(tx_out_file, "w") as out_handle:
-                    for r in _regions_for_coverage(data, region, tx_out_file):
-                        for chrom, start, end, info in _get_coverage_in_region(in_bam, r, depth):
-                            out_handle.write("%s\t%s\t%s\t%s\n" % (chrom, start, end, info))
+            ref_file = tz.get_in(["reference", "fasta", "base"], data)
+            region_file, calc_callable = _regions_for_coverage(data, region, ref_file, tx_out_file)
+            if calc_callable:
+                _group_by_ctype(_get_coverage_file(data["work_bam"], ref_file, region, region_file, depth,
+                                                   tx_out_file, data),
+                                depth, region_file, tx_out_file)
+            # special case, do not calculate if we are in a chromosome not covered by BED file
+            else:
+                os.rename(region_file, tx_out_file)
     return [{"callable_bed": out_file, "config": data["config"], "work_bam": data["work_bam"]}]
 
-def _get_coverage_in_region(in_bam, region, depth):
+def _group_by_ctype(bed_file, depth, region_file, out_file):
+    """Group adjacent callable/uncallble regions into defined intervals.
+
+    Uses tips from bedtools discussion:
+    https://groups.google.com/d/msg/bedtools-discuss/qYDE6XF-GRA/2icQtUeOX_UJ
+    https://gist.github.com/arq5x/b67196a46db5b63bee06
+    """
+    def assign_coverage(feat):
+        feat.name = _get_ctype(float(feat.name), depth)
+        return feat
+    full_out_file = "%s-full%s" % utils.splitext_plus(out_file)
+    with open(full_out_file, "w") as out_handle:
+        for line in open(pybedtools.BedTool(bed_file).each(assign_coverage)
+                                                     .groupby(g=[1, 4], c=[1, 2, 3, 4],
+                                                              ops=["first", "first", "max", "first"]).fn):
+            out_handle.write("\t".join(line.split("\t")[2:]))
+    pybedtools.BedTool(full_out_file).intersect(region_file).saveas(out_file)
+
+def _get_coverage_file(in_bam, ref_file, region, region_file, depth, base_file, data):
     """Retrieve summary of coverage in a region.
     Requires positive non-zero mapping quality at a position, matching GATK's
     CallableLoci defaults.
-
-    Uses chajo's approach of pre-allocating a numpy array for coverage before collapsing
-    to cleanly handle non-covered positions. This uses 2Gb memory for human chr1. If memory
-    requirements become an issue, could look at splitting long chromosomes at smart places.
-
-    XXX Can replace `positions` with chanjo functionality when it accepts max_depth keyword
-    and handles mapping quality.
     """
-    # special case, do not calculate if we are in a chromosome not covered by BED file
-    if region.attrs.get("no_coverage"):
-        yield region.chrom, region.start, region.end, "NO_COVERAGE"
-    else:
-        positions = numpy.zeros(region.end + 1 - region.start)
-        for col in in_bam.pileup(str(region.chrom), region.start, region.end + 1, stepper="all",
-                                 max_depth=depth["max"] + 10, truncate=True):
-            n = sum(1 for read in col.pileups if read.alignment.mapq > 0)
-            positions[col.pos - region.start] = n
-        cur_ctype = None
-        cur_start = None
-        for i, count in enumerate(positions):
-            ctype = _get_ctype(count, depth)
-            if cur_ctype is None:
-                cur_ctype = ctype
-                cur_start = i
-            elif cur_ctype != ctype:
-                yield region.chrom, region.start + cur_start, region.start + i, cur_ctype
-                cur_ctype = ctype
-                cur_start = i
-        if i > cur_start:
-            yield region.chrom, region.start + cur_start, region.start + i, cur_ctype
+    out_file = "%s-genomecov.bed" % utils.splitext_plus(base_file)[0]
+    if not utils.file_exists(out_file):
+        with file_transaction(out_file) as tx_out_file:
+            bam.index(in_bam, data["config"])
+            fai_file = ref.fasta_idx(ref_file, data["config"])
+            sambamba = config_utils.get_program("sambamba", data["config"])
+            bedtools = config_utils.get_program("bedtools", data["config"])
+            max_depth = depth["max"] + 1
+            cmd = ("{sambamba} view -F 'mapping_quality > 0' -L {region_file} -f bam -l 1 {in_bam} | "
+                   "{bedtools} genomecov -ibam stdin -bga -g {fai_file} -max {max_depth} "
+                   "> {tx_out_file}")
+            do.run(cmd.format(**locals()), "bedtools genomecov: %s" % (str(region)), data)
+    # Empty output file, no coverage for the whole contig
+    if not utils.file_exists(out_file):
+        with file_transaction(out_file) as tx_out_file:
+            with open(tx_out_file, "w") as out_handle:
+                for feat in get_ref_bedtool(ref_file, data["config"], region):
+                    out_handle.write("%s\t%s\t%s\t%s\n" % (feat.chrom, feat.start, feat.end, 0))
+    return out_file
 
 def _get_ctype(count, depth):
     if count == 0:
@@ -112,44 +127,46 @@ def _get_ctype(count, depth):
     else:
         return "CALLABLE"
 
-def _regions_for_coverage(data, region, out_file):
-    """Retrieve BedTool iterator over regions we need to calculate coverage in.
+def _regions_for_coverage(data, region, ref_file, out_file):
+    """Retrieve BED file of regions we need to calculate coverage in.
     """
-    variant_regions = utils.get_in(data, ("config", "algorithm", "variant_regions"))
+    variant_regions = bedutils.merge_overlaps(utils.get_in(data, ("config", "algorithm", "variant_regions")),
+                                              data)
     ready_region = shared.subset_variant_regions(variant_regions, region, out_file)
+    custom_file = "%s-coverageregions.bed" % utils.splitext_plus(out_file)[0]
     if not ready_region:
-        return get_ref_bedtool(data["sam_ref"], data["config"])
+        get_ref_bedtool(ref_file, data["config"]).saveas(custom_file)
+        return custom_file, True
     elif os.path.isfile(ready_region):
-        return pybedtools.BedTool(ready_region).intervals
+        return ready_region, True
     elif isinstance(ready_region, (list, tuple)):
         c, s, e = ready_region
-        return [pybedtools.Interval(c, s, e)]
+        pybedtools.BedTool("%s\t%s\t%s\n" % (c, s, e), from_string=True).saveas(custom_file)
+        return custom_file, True
     else:
-        assert isinstance(ready_region, basestring)
-        out = []
-        for r in [x for x in get_ref_bedtool(data["sam_ref"], data["config"])
-                  if x.chrom == ready_region]:
-            # If we have variant regions but none in this region, don't calculate coverage
-            r.attrs["no_coverage"] = variant_regions is not None
-            out.append(r)
-        return out
+        with file_transaction(custom_file) as tx_out_file:
+            with open(tx_out_file, "w") as out_handle:
+                for feat in get_ref_bedtool(ref_file, data["config"], region):
+                    out_handle.write("%s\t%s\t%s\t%s\n" % (feat.chrom, feat.start, feat.end, "NO_COVERAGE"))
+        return custom_file, variant_regions is None
 
 def sample_callable_bed(bam_file, ref_file, config):
     """Retrieve callable regions for a sample subset by defined analysis regions.
     """
     out_file = "%s-callable_sample.bed" % os.path.splitext(bam_file)[0]
-    callable_bed = parallel_callable_loci(bam_file, ref_file, config)
-    input_regions_bed = config["algorithm"].get("variant_regions", None)
-    if not utils.file_uptodate(out_file, callable_bed):
-        with file_transaction(out_file) as tx_out_file:
-            callable_regions = pybedtools.BedTool(callable_bed)
-            filter_regions = callable_regions.filter(lambda x: x.name == "CALLABLE")
-            if input_regions_bed:
-                if not utils.file_uptodate(out_file, input_regions_bed):
-                    input_regions = pybedtools.BedTool(input_regions_bed)
-                    filter_regions.intersect(input_regions).saveas(tx_out_file)
-            else:
-                filter_regions.saveas(tx_out_file)
+    with shared.bedtools_tmpdir({"config": config}):
+        callable_bed = parallel_callable_loci(bam_file, ref_file, config)
+        input_regions_bed = config["algorithm"].get("variant_regions", None)
+        if not utils.file_uptodate(out_file, callable_bed):
+            with file_transaction(out_file) as tx_out_file:
+                callable_regions = pybedtools.BedTool(callable_bed)
+                filter_regions = callable_regions.filter(lambda x: x.name == "CALLABLE")
+                if input_regions_bed:
+                    if not utils.file_uptodate(out_file, input_regions_bed):
+                        input_regions = pybedtools.BedTool(input_regions_bed)
+                        filter_regions.intersect(input_regions).saveas(tx_out_file)
+                else:
+                    filter_regions.saveas(tx_out_file)
     return out_file
 
 def get_ref_bedtool(ref_file, config, chrom=None):
@@ -227,6 +244,7 @@ class NBlockRegionPicker:
     maintain the splitting.
     """
     def __init__(self, ref_regions, config):
+        self._end_buffer = 250
         self._chr_last_blocks = {}
         target_blocks = int(config["algorithm"].get("nomap_split_targets", 2000))
         self._target_size = self._get_target_size(target_blocks, ref_regions)
@@ -242,15 +260,29 @@ class NBlockRegionPicker:
         """Check for inclusion of block based on distance from previous.
         """
         last_pos = self._chr_last_blocks.get(x.chrom, 0)
-        if (x.start - last_pos) > self._target_size:
-            self._chr_last_blocks[x.chrom] = x.stop
+        # Region excludes an entire chromosome, typically decoy/haplotypes
+        if last_pos < self._end_buffer and x.stop >= self._ref_sizes.get(x.chrom, 0) - self._end_buffer:
             return True
-        # fills an entire chromosome, handles smaller decoy and haplotype chromosomes
-        elif last_pos == 0 and x.stop >= self._ref_sizes.get(x.chrom, 0) - 100:
+        # Do not split on smaller decoy and haplotype chromosomes
+        elif self._ref_sizes.get(x.chrom, 0) <= self._target_size:
+            return False
+        elif (x.start - last_pos) > self._target_size:
+            self._chr_last_blocks[x.chrom] = x.stop
             return True
         else:
             return False
 
+    def expand_block(self, feat):
+        """Expand any blocks which are near the start or end of a contig.
+        """
+        chrom_end = self._ref_sizes.get(feat.chrom)
+        if chrom_end:
+            if feat.start < self._end_buffer:
+                feat.start = 0
+            if feat.stop >= chrom_end - self._end_buffer:
+                feat.stop = chrom_end
+        return feat
+
 def block_regions(in_bam, ref_file, config):
     """Find blocks of regions for analysis from mapped input BAM file.
 
@@ -258,23 +290,25 @@ def block_regions(in_bam, ref_file, config):
     with no read support, that can be analyzed independently.
     """
     min_n_size = int(config["algorithm"].get("nomap_split_size", 100))
-    callable_bed = parallel_callable_loci(in_bam, ref_file, config)
-    nblock_bed = "%s-nblocks%s" % os.path.splitext(callable_bed)
-    callblock_bed = "%s-callableblocks%s" % os.path.splitext(callable_bed)
-    if not utils.file_uptodate(nblock_bed, callable_bed):
-        ref_regions = get_ref_bedtool(ref_file, config)
-        nblock_regions = _get_nblock_regions(callable_bed, min_n_size)
-        nblock_regions = _add_config_regions(nblock_regions, ref_regions, config)
-        nblock_regions.saveas(nblock_bed)
-        if len(ref_regions.subtract(nblock_regions)) > 0:
-            ref_regions.subtract(nblock_bed).merge(d=min_n_size).saveas(callblock_bed)
-        else:
-            raise ValueError("No callable regions found from BAM file. Alignment regions might "
-                             "not overlap with regions found in your `variant_regions` BED: %s" % in_bam)
+    with shared.bedtools_tmpdir({"config": config}):
+        callable_bed = parallel_callable_loci(in_bam, ref_file, config)
+        nblock_bed = "%s-nblocks%s" % os.path.splitext(callable_bed)
+        callblock_bed = "%s-callableblocks%s" % os.path.splitext(callable_bed)
+        if not utils.file_uptodate(nblock_bed, callable_bed):
+            ref_regions = get_ref_bedtool(ref_file, config)
+            nblock_regions = _get_nblock_regions(callable_bed, min_n_size)
+            nblock_regions = _add_config_regions(nblock_regions, ref_regions, config)
+            nblock_regions.saveas(nblock_bed)
+            if len(ref_regions.subtract(nblock_regions)) > 0:
+                ref_regions.subtract(nblock_bed).merge(d=min_n_size).saveas(callblock_bed)
+            else:
+                raise ValueError("No callable regions found from BAM file. Alignment regions might "
+                                 "not overlap with regions found in your `variant_regions` BED: %s" % in_bam)
     return callblock_bed, nblock_bed, callable_bed
 
-def _write_bed_regions(sample, final_regions, out_file, out_file_ref):
-    ref_regions = get_ref_bedtool(sample["sam_ref"], sample["config"])
+def _write_bed_regions(data, final_regions, out_file, out_file_ref):
+    ref_file = tz.get_in(["reference", "fasta", "base"], data)
+    ref_regions = get_ref_bedtool(ref_file, data["config"])
     noanalysis_regions = ref_regions.subtract(final_regions)
     final_regions.saveas(out_file)
     noanalysis_regions.saveas(out_file_ref)
@@ -351,20 +385,26 @@ def combine_sample_regions(*samples):
         global_analysis_file = None
     out = []
     analysis_files = []
-    for batch, items in vmulti.group_by_batch(samples).items():
-        if global_analysis_file:
-            analysis_file, no_analysis_file = global_analysis_file, global_no_analysis_file
-        else:
-            analysis_file, no_analysis_file = _combine_sample_regions_batch(batch, items)
-        for data in items:
-            if analysis_file:
-                analysis_files.append(analysis_file)
-                data["config"]["algorithm"]["callable_regions"] = analysis_file
-                data["config"]["algorithm"]["non_callable_regions"] = no_analysis_file
-            out.append([data])
-    assert len(out) == len(samples)
-    final_regions = pybedtools.BedTool(analysis_files[0])
-    _analysis_block_stats(final_regions)
+    with shared.bedtools_tmpdir(samples[0]):
+        for batch, items in vmulti.group_by_batch(samples).items():
+            if global_analysis_file:
+                analysis_file, no_analysis_file = global_analysis_file, global_no_analysis_file
+            else:
+                analysis_file, no_analysis_file = _combine_sample_regions_batch(batch, items)
+            for data in items:
+                vr_file = tz.get_in(["config", "algorithm", "variant_regions"], data)
+                if analysis_file:
+                    analysis_files.append(analysis_file)
+                    data["config"]["algorithm"]["callable_regions"] = analysis_file
+                    data["config"]["algorithm"]["non_callable_regions"] = no_analysis_file
+                    data["config"]["algorithm"]["callable_count"] = pybedtools.BedTool(analysis_file).count()
+                elif vr_file:
+                    data["config"]["algorithm"]["callable_count"] = pybedtools.BedTool(vr_file).count()
+                out.append([data])
+        assert len(out) == len(samples)
+        if len(analysis_files) > 0:
+            final_regions = pybedtools.BedTool(analysis_files[0])
+            _analysis_block_stats(final_regions)
     return out
 
 def _combine_sample_regions_batch(batch, items):
@@ -384,18 +424,15 @@ def _combine_sample_regions_batch(batch, items):
             analysis_file, no_analysis_file = None, None
         else:
             nblock_regions = reduce(operator.add, bed_regions)
-            ref_regions = get_ref_bedtool(items[0]["sam_ref"], config)
+            ref_file = tz.get_in(["reference", "fasta", "base"], items[0])
+            ref_regions = get_ref_bedtool(ref_file, config)
             min_n_size = int(config["algorithm"].get("nomap_split_size", 100))
             ec_regions = _combine_excessive_coverage(items, ref_regions, min_n_size)
-            block_filter = NBlockRegionPicker(ref_regions, config)
-            nblock_size_filtered = nblock_regions.filter(block_filter.include_block).saveas()
-            if len(nblock_size_filtered) >= len(ref_regions):
-                final_nblock_regions = nblock_size_filtered
-            else:
-                final_nblock_regions = nblock_regions
-            final_regions = ref_regions.subtract(final_nblock_regions)
             if len(ec_regions) > 0:
-                final_regions = final_regions.subtract(ec_regions)
-            final_regions.merge(d=min_n_size)
+                nblock_regions = nblock_regions.cat(ec_regions, d=min_n_size)
+            block_filter = NBlockRegionPicker(ref_regions, config)
+            final_nblock_regions = nblock_regions.filter(
+                block_filter.include_block).each(block_filter.expand_block).saveas()
+            final_regions = ref_regions.subtract(final_nblock_regions).merge(d=min_n_size)
             _write_bed_regions(items[0], final_regions, analysis_file, no_analysis_file)
     return analysis_file, no_analysis_file
