@@ -8,7 +8,11 @@ Cluster implementation from ipython-cluster-helper:
 
 https://github.com/roryk/ipython-cluster-helper
 """
+import json
 import os
+import zlib
+
+import toolz as tz
 
 from bcbio import utils
 from bcbio.log import logger, get_log_dir
@@ -38,6 +42,37 @@ def _get_ipython_fn(fn_name, parallel):
                               fromlist=["ipythontasks"]),
                    import_fn_name)
 
+def zip_args(items, config):
+    """Compress and JSON encode arguments before sending to IPython, if configured.
+    """
+    if tz.get_in(["algorithm", "compress_msg"], config):
+        #print [len(json.dumps(x)) for x in items]
+        items = [zlib.compress(json.dumps(x, separators=(',', ':')), 9) for x in items]
+        #print [len(x) for x in items]
+    return items
+
+def unzip_args(args):
+    """Unzip arguments if passed as compressed JSON string.
+
+    Checks string if zlib compressed to handle zipped and unzipped cases:
+    http://stackoverflow.com/questions/5322860/how-to-detect-quickly-if-a-string-is-zlib-compressed
+    https://github.com/mgoldfar/ECE595-WikiTrace/blob/master/WikiTrace.py#L66
+    """
+    def _is_zlib_compressed(arg):
+        if not arg or not isinstance(arg, basestring) or len(arg) < 2:
+            return False
+        cmf = ord(arg[0])
+        if cmf & 0x0f != 0x08 and cmf & 0xf0 != 0x70:
+            return False
+        flg = ord(arg[1])
+        if (cmf * 256 + flg) % 31 != 0:
+            return False
+        return True
+    if len(args) > 0 and all(_is_zlib_compressed(arg) for arg in args):
+        return [json.loads(zlib.decompress(arg)) for arg in args]
+    else:
+        return args
+
 def runner(view, parallel, dirs, config):
     """Run a task on an ipython parallel cluster, allowing alternative queue types.
 
@@ -54,8 +89,9 @@ def runner(view, parallel, dirs, config):
             if "wrapper" in parallel:
                 wrap_parallel = {k: v for k, v in parallel.items() if k in set(["fresources"])}
                 items = [[fn_name] + parallel.get("wrapper_args", []) + [wrap_parallel] + list(x) for x in items]
+            items = zip_args(items, config)
             for data in view.map_sync(fn, items, track=False):
                 if data:
-                    out.extend(data)
+                    out.extend(unzip_args(data))
         return out
     return run
