@@ -2,27 +2,33 @@
 """
 from bcbio import structural, utils, chipseq
 from bcbio.bam import callable
+from bcbio.rnaseq import sailfish
 from bcbio.ngsalign import alignprep
-from bcbio.pipeline import (archive, disambiguate, lane, qcsummary, sample, shared, variation,
-                            rnaseq)
-from bcbio.variation import (bamprep, bedutils, coverage, genotype, ensemble, multi, population,
-                             recalibrate, validate, vcfutils)
+from bcbio.pipeline import (archive, disambiguate, qcsummary, sample,
+                            shared, variation, rnaseq)
+from bcbio.variation import (bamprep, bedutils, coverage, genotype, ensemble,
+                             joint, multi, population, recalibrate, validate,
+                             vcfutils)
 
 @utils.map_wrap
-def process_lane(*args):
-    return lane.process_lane(*args)
+def run_sailfish(*args):
+    return sailfish.run_sailfish(*args)
 
 @utils.map_wrap
-def trim_lane(*args):
-    return lane.trim_lane(*args)
+def prepare_sample(*args):
+    return sample.prepare_sample(*args)
+
+@utils.map_wrap
+def trim_sample(*args):
+    return sample.trim_sample(*args)
 
 @utils.map_wrap
 def process_alignment(*args):
-    return lane.process_alignment(*args)
+    return sample.process_alignment(*args)
 
 @utils.map_wrap
 def postprocess_alignment(*args):
-    return lane.postprocess_alignment(*args)
+    return sample.postprocess_alignment(*args)
 
 @utils.map_wrap
 def prep_align_inputs(*args):
@@ -127,3 +133,15 @@ def clean_chipseq_alignment(*args):
 @utils.map_wrap
 def archive_to_cram(*args):
     return archive.to_cram(*args)
+
+@utils.map_wrap
+def square_batch_region(*args):
+    return joint.square_batch_region(*args)
+
+@utils.map_wrap
+def cufflinks_assemble(*args):
+    return rnaseq.cufflinks_assemble(*args)
+
+@utils.map_wrap
+def cufflinks_merge(*args):
+    return rnaseq.cufflinks_merge(*args)
