@@ -58,13 +58,19 @@ def upgrade_bcbio(args):
     if args.tooldir:
         with bcbio_tmpdir():
             print("Upgrading third party tools to latest versions")
+            _symlink_bcbio(args)
             upgrade_thirdparty_tools(args, REMOTES)
             print("Third party tools upgrade complete.")
     if args.install_data:
-        with bcbio_tmpdir():
-            print("Upgrading bcbio-nextgen data files")
-            upgrade_bcbio_data(args, REMOTES)
-            print("bcbio-nextgen data upgrade complete.")
+        if len(args.genomes) == 0:
+            print("Data not installed, no genomes provided with `--genomes` flag")
+        elif len(args.aligners) == 0:
+            print("Data not installed, no aligners provided with `--aligners` flag")
+        else:
+            with bcbio_tmpdir():
+                print("Upgrading bcbio-nextgen data files")
+                upgrade_bcbio_data(args, REMOTES)
+                print("bcbio-nextgen data upgrade complete.")
     if args.isolate and args.tooldir:
         print("Installation directory not added to current PATH")
         print("  Add {t}/bin to PATH and {t}/lib to LD_LIBRARY_PATH".format(t=args.tooldir))
@@ -74,6 +80,17 @@ def upgrade_bcbio(args):
     print("Upgrade completed successfully.")
     return args
 
+def _symlink_bcbio(args):
+    """Ensure bcbio_nextgen.py symlink in final tool directory.
+    """
+    bcbio_anaconda = os.path.join(os.path.dirname(sys.executable), "bcbio_nextgen.py")
+    bcbio_final = os.path.join(args.tooldir, "bin", "bcbio_nextgen.py")
+    sudo_cmd = ["sudo"] if args.sudo else []
+    if not os.path.exists(bcbio_final):
+        if os.path.lexists(bcbio_final):
+            subprocess.check_call(sudo_cmd + ["rm", "-f", bcbio_final])
+        subprocess.check_call(sudo_cmd + ["ln", "-s", bcbio_anaconda, bcbio_final])
+
 def _install_container_bcbio_system(datadir):
     """Install limited bcbio_system.yaml file for setting core and memory usage.
 
@@ -124,11 +141,11 @@ def _update_conda_packages():
     """If installed in an anaconda directory, upgrade conda packages.
     """
     conda_bin = os.path.join(os.path.dirname(sys.executable), "conda")
-    pkgs = ["biopython", "boto", "cython", "ipython", "lxml", "matplotlib",
-            "nose", "numpy", "pandas", "patsy", "pycrypto", "pip", "pysam",
-            "pyyaml", "pyzmq", "requests", "scipy", "setuptools", "sqlalchemy",
-            "statsmodels", "toolz", "tornado"]
-    channels = ["-c", "https://conda.binstar.org/collections/chapmanb/bcbio"]
+    pkgs = ["biopython", "boto", "cpat", "cython", "ipython", "lxml",
+            "matplotlib", "nose", "numpy", "pandas", "patsy", "pycrypto",
+            "pip", "pysam", "pyyaml", "pyzmq", "requests", "scipy",
+            "setuptools", "sqlalchemy", "statsmodels", "toolz", "tornado"]
+    channels = ["-c", "https://conda.binstar.org/bcbio"]
     if os.path.exists(conda_bin):
         subprocess.check_call([conda_bin, "install", "--yes", "numpy"])
         subprocess.check_call([conda_bin, "install", "--yes"] + channels + pkgs)
@@ -141,12 +158,22 @@ def _get_data_dir():
                          "located in the same directory as `galaxy` `genomes` and `gemini_data` directories.")
     return os.path.dirname(base_dir)
 
+def get_gemini_dir():
+    try:
+        data_dir = _get_data_dir()
+        return os.path.join(data_dir, "gemini_data")
+    except ValueError:
+        return None
+
 def upgrade_bcbio_data(args, remotes):
     """Upgrade required genome data files in place.
     """
     data_dir = _get_data_dir()
     s = _default_deploy_args(args)
     s["actions"] = ["setup_biodata"]
+    tooldir = args.tooldir or get_defaults().get("tooldir")
+    if tooldir:
+        s["fabricrc_overrides"]["system_install"] = tooldir
     s["fabricrc_overrides"]["data_files"] = data_dir
     s["fabricrc_overrides"]["galaxy_home"] = os.path.join(data_dir, "galaxy")
     cbl = get_cloudbiolinux(remotes)
@@ -157,9 +184,14 @@ def upgrade_bcbio_data(args, remotes):
     _upgrade_genome_resources(s["fabricrc_overrides"]["galaxy_home"],
                               remotes["genome_resources"])
     _upgrade_snpeff_data(s["fabricrc_overrides"]["galaxy_home"], args, remotes)
-    if 'data' in set([x.name for x in args.toolplus]):
+    _upgrade_vep_data(s["fabricrc_overrides"]["galaxy_home"])
+    toolplus = set([x.name for x in args.toolplus])
+    if 'data' in toolplus:
         gemini = os.path.join(os.path.dirname(sys.executable), "gemini")
-        subprocess.check_call([gemini, "update", "--dataonly"])
+        extras = []
+        if "cadd" in toolplus:
+            extras.extend(["--extra", "cadd_score"])
+        subprocess.check_call([gemini, "update", "--dataonly"] + extras)
 
 def _upgrade_genome_resources(galaxy_dir, base_url):
     """Retrieve latest version of genome resource YAML configuration files.
@@ -184,6 +216,10 @@ def _upgrade_genome_resources(galaxy_dir, base_url):
                 with open(local_file, "w") as out_handle:
                     out_handle.write(r.text)
 
+def _upgrade_vep_data(galaxy_dir):
+    for dbkey, ref_file in genome.get_builds(galaxy_dir):
+        effects.prep_vep_cache(dbkey, ref_file)
+
 def _upgrade_snpeff_data(galaxy_dir, args, remotes):
     """Install or upgrade snpEff databases, localized to reference directory.
     """
@@ -198,14 +234,7 @@ def _upgrade_snpeff_data(galaxy_dir, args, remotes):
                 snpeff_db_dir = os.path.join(snpeff_base_dir, snpeff_db)
                 if not os.path.exists(snpeff_db_dir):
                     print("Installing snpEff database %s in %s" % (snpeff_db, snpeff_base_dir))
-                    tooldir = args.tooldir or get_defaults()["tooldir"]
-                    config = {"resources": {"snpeff": {"jvm_opts": ["-Xms500m", "-Xmx1g"],
-                                                       "dir": os.path.join(tooldir, "share", "java", "snpeff")}}}
-                    raw_version = programs.java_versioner("snpeff", "snpEff",
-                                                          stdout_flag="snpEff version SnpEff")(config)
-                    snpeff_version = "".join([x for x in raw_version
-                                              if x in set(string.digits + ".")]).replace(".", "_")
-                    dl_url = remotes["snpeff_dl_url"].format(snpeff_ver=snpeff_version, genome=snpeff_db)
+                    dl_url = remotes["snpeff_dl_url"].format(snpeff_ver=_get_snpeff_version(args), genome=snpeff_db)
                     dl_file = os.path.basename(dl_url)
                     with utils.chdir(snpeff_base_dir):
                         subprocess.check_call(["wget", "-c", "-O", dl_file, dl_url])
@@ -215,14 +244,35 @@ def _upgrade_snpeff_data(galaxy_dir, args, remotes):
                     os.rename(dl_dir, snpeff_db_dir)
                     os.rmdir(os.path.join(snpeff_base_dir, "data"))
 
+def _get_snpeff_version(args):
+    tooldir = args.tooldir or get_defaults()["tooldir"]
+    raw_version = programs.get_version_manifest("snpeff")
+    if not raw_version:
+        config = {"resources": {"snpeff": {"jvm_opts": ["-Xms500m", "-Xmx1g"],
+                                           "dir": os.path.join(tooldir, "share", "java", "snpeff")}}}
+        raw_version = programs.java_versioner("snpeff", "snpEff",
+                                              stdout_flag="snpEff version SnpEff")(config)
+    snpeff_version = "".join([x for x in raw_version
+                              if x in set(string.digits + ".")]).replace(".", "_")
+    assert snpeff_version, "Did not find snpEff version information"
+    return snpeff_version
+
 def _get_biodata(base_file, args):
     with open(base_file) as in_handle:
         config = yaml.load(in_handle)
     config["install_liftover"] = False
     config["genome_indexes"] = args.aligners
-    config["genomes"] = [g for g in config["genomes"] if g["dbkey"] in args.genomes]
+    config["genomes"] = [_add_biodata_flags(g, args) for g in config["genomes"] if g["dbkey"] in args.genomes]
     return config
 
+def _add_biodata_flags(g, args):
+    toolplus = set([x.name for x in args.toolplus])
+    if g["dbkey"] in ["hg19", "GRCh37"]:
+        for flag in ["dbnsfp"]:
+            if flag in toolplus:
+                g[flag] = True
+    return g
+
 def upgrade_thirdparty_tools(args, remotes):
     """Install and update third party tools used in the pipeline.
 
@@ -261,10 +311,14 @@ def _install_toolplus(args, manifest_dir):
     for tool in args.toolplus:
         if tool.name == "data":
             _install_gemini(args.tooldir, _get_data_dir(), args)
+        elif tool.name == "kraken":
+            _install_kraken_db(tool. fname, _get_data_dir(), args)
         elif tool.name in set(["gatk", "mutect"]):
             _install_gatk_jar(tool.name, tool.fname, toolplus_manifest, system_config, toolplus_dir)
         elif tool.name in set(["protected"]):  # back compatibility
             pass
+        elif tool.name in set(["cadd", "dbnsfp"]):  # larger data targets
+            pass
         else:
             raise ValueError("Unexpected toolplus argument: %s %s" (tool.name, tool.fname))
 
@@ -338,6 +392,29 @@ def _install_gemini(tooldir, datadir, args):
         subprocess.check_call(cmd)
         os.remove(script)
 
+def _install_kraken_db(fname, datadir, args):
+    """Install kraken minimal DB in genome folder.
+    """
+    kraken = os.path.join(datadir, "genome/kraken")
+    url = "https://ccb.jhu.edu/software/kraken/dl/minikraken.tgz"
+    compress = os.path.join(kraken, os.path.basename(url))
+    base, ext = utils.splitext_plus(os.path.basename(url))
+    db = os.path.join(kraken, base)
+    tooldir = args.tooldir or get_defaults()["tooldir"]
+    if os.path.exists(os.path.join(tooldir, "bin", "kraken")):
+        if not os.path.exists(kraken):
+            utils.safe_makedir(kraken)
+        if not os.path.exists(db):
+            if not os.path.exists(compress):
+                subprocess.check_call(["wget", "-O", compress, url, "--no-check-certificate"])
+            cmd = ["tar", "-xzvf", compress, "-C", kraken]
+            subprocess.check_call(cmd)
+            shutil.move(os.path.join(kraken, "minikraken_20140330"), os.path.join(kraken, "minikraken"))
+            utils.remove_safe(compress)
+    else:
+        raise argparse.ArgumentTypeError("kraken not installed in tooldir %s." %
+                                         os.path.join(tooldir, "bin", "kraken"))
+
 # ## Store a local configuration file with upgrade details
 
 def _get_install_config():
@@ -421,7 +498,7 @@ def get_defaults():
 def _check_toolplus(x):
     """Parse options for adding non-standard/commercial tools like GATK and MuTecT.
     """
-    std_choices = set(["data"])
+    std_choices = set(["data", "cadd", "dbnsfp", "kraken"])
     if x in std_choices:
         return Tool(x, None)
     elif "=" in x and len(x.split("=")) == 2:
@@ -447,15 +524,16 @@ def add_subparser(subparsers):
     parser.add_argument("--toolplus", help="Specify additional tool categories to install",
                         action="append", default=[], type=_check_toolplus)
     parser.add_argument("--genomes", help="Genomes to download",
-                        action="append", default=["GRCh37"],
-                        choices=["GRCh37", "hg19", "mm10", "mm9", "rn5", "canFam3", "dm3", "Zv9", "phix"])
+                        action="append", default=[],
+                        choices=["GRCh37", "hg19", "mm10", "mm9", "rn5", "canFam3", "dm3", "Zv9", "phix", "sacCer3",
+                                 "xenTro3", "TAIR10", "WBcel235"])
     parser.add_argument("--aligners", help="Aligner indexes to download",
-                        action="append", default=["bwa", "bowtie2"],
+                        action="append", default=[],
                         choices=["bowtie", "bowtie2", "bwa", "novoalign", "star", "ucsc"])
     parser.add_argument("--data", help="Upgrade data dependencies",
                         dest="install_data", action="store_true", default=False)
-    parser.add_argument("--nosudo", help="Specify we cannot use sudo for commands",
-                        dest="sudo", action="store_false", default=True)
+    parser.add_argument("--sudo", help="Use sudo for the installation, enabling install of system packages",
+                        dest="sudo", action="store_true", default=False)
     parser.add_argument("--isolate", help="Created an isolated installation without PATH updates",
                         dest="isolate", action="store_true", default=False)
     parser.add_argument("--distribution", help="Operating system distribution",
