@@ -49,6 +49,8 @@ def load_system_config(config_file, work_dir=None):
     installs (galaxy/bcbio_system.yaml) and docker installs (config/bcbio_system.yaml).
     """
     docker_config = _get_docker_config()
+    if config_file is None:
+        config_file = "bcbio_system.yaml"
     if not os.path.exists(config_file):
         base_dir = get_base_installdir()
         test_config = os.path.join(base_dir, "galaxy", config_file)
@@ -95,7 +97,7 @@ def _merge_system_configs(host_config, container_config, out_file=None):
                                                              "universe_wsgi.ini"))
     if out_file:
         with open(out_file, "w") as out_handle:
-            yaml.dump(out, out_handle, default_flow_style=False, allow_unicode=False)
+            yaml.safe_dump(out, out_handle, default_flow_style=False, allow_unicode=False)
     return out
 
 def _get_docker_config():
@@ -262,6 +264,18 @@ def get_algorithm_config(xs):
     raise ValueError("Did not find algorithm configuration in items: {0}"
                      .format(xs))
 
+def get_dataarg(args):
+    """Retrieve the world 'data' argument from a set of input parameters.
+    """
+    for i, arg in enumerate(args):
+        if is_nested_config_arg(arg):
+            return i, arg
+        elif is_std_config_arg(arg):
+            return i, {"config": arg}
+        elif isinstance(arg, (list, tuple)) and is_nested_config_arg(arg[0]):
+            return i, arg[0]
+    raise ValueError("Did not find configuration or data object in arguments: %s" % args)
+
 def add_cores_to_config(args, cores_per_job, parallel=None):
     """Add information about available cores for a job to configuration.
     Ugly hack to update core information in a configuration dictionary.
@@ -361,11 +375,17 @@ def use_vqsr(algs):
             return True
     return False
 
-## functions for navigating through the standard galaxy directory of files
+def use_snpeff(algs):
+    """Processing uses snpEff. Avoids memory requirements if not used.
+    """
+    return any(alg.get("effects", "snpeff") == "snpeff" and alg.get("variantcaller") for alg in algs)
 
-def get_transcript_gtf(genome_dir):
-    out_file = os.path.join(genome_dir, "rnaseq", "ref-transcripts.gtf")
-    return out_file
+def use_bcbio_variation_recall(algs):
+    """Processing uses bcbio-variation-recall. Avoids core requirement if not used.
+    """
+    return any(alg.get("jointcaller") == "bcbio-variation-recall" for alg in algs)
+
+## functions for navigating through the standard galaxy directory of files
 
 def get_rRNA_interval(genome_dir):
     return os.path.join(genome_dir, "rnaseq", "rRNA.interval_list")
