@@ -47,8 +47,13 @@ def abs_file_paths(xs, base_dir=None, ignore_keys=None):
     os.chdir(base_dir)
     out = {}
     for k, v in xs.iteritems():
-        if k not in ignore_keys and v and isinstance(v, basestring) and os.path.exists(v):
-            out[k] = os.path.normpath(os.path.join(base_dir, v))
+        if k not in ignore_keys and v and isinstance(v, basestring):
+            if v.lower() == "none":
+                out[k] = None
+            elif os.path.exists(v):
+                out[k] = os.path.normpath(os.path.join(base_dir, v))
+            else:
+                out[k] = v
         else:
             out[k] = v
     os.chdir(orig_dir)
@@ -170,9 +175,9 @@ def get_refs(genome_build, aligner, galaxy_base):
                 indexes = glob.glob(os.path.join(base, "*"))
             else:
                 indexes = glob.glob("%s*" % utils.splitext_plus(base)[0])
-            if base in indexes:
-                indexes.remove(base)
-            out[name_remap.get(name, name)] = {"base": base, "indexes": indexes}
+            out[name_remap.get(name, name)] = {"indexes": indexes}
+            if os.path.exists(base) and os.path.isfile(base):
+                out[name_remap.get(name, name)]["base"] = base
     return out
 
 def get_builds(galaxy_base):
