@@ -22,6 +22,7 @@ from bcbio import bam, utils
 from bcbio.distributed.transaction import file_transaction
 from bcbio.log import logger
 from bcbio.pipeline import config_utils, run_info
+from bcbio.install import _get_data_dir
 from bcbio.provenance import do
 import bcbio.rnaseq.qc
 from bcbio.variation.realign import has_aligned_reads
@@ -47,7 +48,7 @@ def pipeline_summary(data):
     """Provide summary information on processing sample.
     """
     work_bam = data.get("work_bam")
-    if data["sam_ref"] is not None and work_bam and has_aligned_reads(work_bam):
+    if data["sam_ref"] is not None and work_bam and work_bam.endswith(".bam"):
         logger.info("Generating summary files: %s" % str(data["name"]))
         data["summary"] = _run_qc_tools(work_bam, data)
     return [[data]]
@@ -79,12 +80,13 @@ def prep_pdf(qc_dir, config):
 def _run_qc_tools(bam_file, data):
     """Run a set of third party quality control tools, returning QC directory and metrics.
     """
+    metrics = {}
     to_run = [("fastqc", _run_fastqc)]
-    if data["analysis"].lower() == "rna-seq":
+    if data["analysis"].lower().startswith("rna-seq"):
         to_run.append(("rnaseqc", bcbio.rnaseq.qc.sample_summary))
 #        to_run.append(("coverage", _run_gene_coverage))
         to_run.append(("complexity", _run_complexity))
-    elif data["analysis"].lower() == "chip-seq":
+    elif data["analysis"].lower().startswith("chip-seq"):
         to_run.append(["bamtools", _run_bamtools_stats])
     else:
         to_run += [("bamtools", _run_bamtools_stats), ("gemini", _run_gemini_stats)]
@@ -94,6 +96,10 @@ def _run_qc_tools(bam_file, data):
         cur_qc_dir = os.path.join(qc_dir, program_name)
         cur_metrics = qc_fn(bam_file, data, cur_qc_dir)
         metrics.update(cur_metrics)
+    ratio = bam.get_aligned_reads(bam_file, data)
+    if ratio < 0.60 and data['config']["algorithm"].get("kraken", False) and data["analysis"].lower() == "rna-seq":
+        cur_metrics = _run_kraken(data, ratio)
+        metrics.update(cur_metrics)
     metrics["Name"] = data["name"][-1]
     metrics["Quality format"] = utils.get_in(data,
                                              ("config", "algorithm",
@@ -129,6 +135,7 @@ def write_project_summary(samples):
                        default_flow_style=False, allow_unicode=False)
     return out_file
 
+
 def _other_pipeline_samples(summary_file, cur_samples):
     """Retrieve samples produced previously by another pipeline in the summary output.
     """
@@ -252,11 +259,72 @@ def _run_gene_coverage(bam_file, data, out_dir):
     return {"gene_coverage": out_file}
 
 
+
+def _run_kraken(data,ratio):
+    """Run kraken, generating report in specified directory and parsing metrics.
+       Using only first paired reads.
+    """
+    logger.info("Number of aligned reads < than 0.60 in %s: %s" % (str(data["name"]),ratio))
+    logger.info("Running kraken to determine contaminant: %s" % str(data["name"]))
+    qc_dir = utils.safe_makedir(os.path.join(data["dirs"]["work"], "qc", data["description"]))
+    kraken_out = os.path.join(qc_dir, "kraken")
+    stats = out = out_stats = None
+    db = data['config']["algorithm"]["kraken"] 
+    if db == "minikraken":
+        db = os.path.join(_get_data_dir(),"genome","kraken","minikraken")
+    else:
+        if not os.path.exists(db):
+            logger.info("kraken: no database found %s, skipping" % db)
+            return {"kraken_report" : "null"}
+    if not os.path.exists(os.path.join(kraken_out,"kraken_out")):
+        work_dir = os.path.dirname(kraken_out)
+        utils.safe_makedir(work_dir)
+        num_cores = data["config"]["algorithm"].get("num_cores", 1)
+        files = data["files"]        
+        with utils.curdir_tmpdir(data, work_dir) as tx_tmp_dir:
+            with utils.chdir(tx_tmp_dir):
+                out = os.path.join(tx_tmp_dir,"kraken_out")
+                out_stats = os.path.join(tx_tmp_dir,"kraken_stats")
+                cl = (" ").join([config_utils.get_program("kraken", data["config"]),
+                      "--db",db,"--quick",
+                      "--preload","--min-hits","2","--threads",str(num_cores), 
+                      "--out", out, files[0]," 2>",out_stats])
+                do.run(cl,"kraken: %s" % data["name"][-1])
+                if os.path.exists(kraken_out):
+                    shutil.rmtree(kraken_out)
+                shutil.move(tx_tmp_dir, kraken_out)
+    metrics = _parse_kraken_output(kraken_out,db,data)
+    return metrics
+
+def _parse_kraken_output(out_dir, db, data):
+    """Parse kraken stat info comming from stderr, 
+       generating report with kraken-report
+    """
+    in_file = os.path.join(out_dir,"kraken_out")
+    stat_file = os.path.join(out_dir,"kraken_stats")
+    out_file = os.path.join(out_dir, "kraken_summary")
+    classify = unclassify = None
+    with open(stat_file,'r') as handle:
+        for line in handle:
+            if line.find(" classified") > -1:
+                classify = line[line.find("(")+1:line.find(")")]
+            if line.find(" unclassified") > -1:
+                unclassify = line[line.find("(")+1:line.find(")")]
+    if os.path.getsize(in_file)>0:
+        with file_transaction(out_file) as tx_out_file:
+            cl = (" ").join([config_utils.get_program("kraken-report", data["config"]),
+                          "--db",db,in_file,">",tx_out_file])
+            do.run(cl, "kraken report: %s" % data["name"][-1])
+    return {"kraken_report" : out_file,"kraken_clas": classify,"kraken_unclas": unclassify}
+
 def _run_fastqc(bam_file, data, fastqc_out):
     """Run fastqc, generating report in specified directory and parsing metrics.
 
     Downsamples to 10 million reads to avoid excessive processing times with large
     files, unless we're running a Standard/QC pipeline.
+
+    Handles fastqc 0.11+, which use a single HTML file and older versions that use
+    a directory of files + images. The goal is to eventually move to only 0.11+
     """
     sentry_file = os.path.join(fastqc_out, "fastqc_report.html")
     if not os.path.exists(sentry_file):
@@ -266,20 +334,25 @@ def _run_fastqc(bam_file, data, fastqc_out):
                   if data.get("analysis", "").lower() not in ["standard"]
                   else None)
         bam_file = ds_bam if ds_bam else bam_file
+        fastqc_name = os.path.splitext(os.path.basename(bam_file))[0]
         num_cores = data["config"]["algorithm"].get("num_cores", 1)
         with utils.curdir_tmpdir(data, work_dir) as tx_tmp_dir:
             with utils.chdir(tx_tmp_dir):
                 cl = [config_utils.get_program("fastqc", data["config"]),
-                      "-t", str(num_cores), "-o", tx_tmp_dir, "-f", "bam", bam_file]
+                      "-t", str(num_cores), "--extract", "-o", tx_tmp_dir, "-f", "bam", bam_file]
                 do.run(cl, "FastQC: %s" % data["name"][-1])
-                fastqc_outdir = os.path.join(tx_tmp_dir,
-                                             "%s_fastqc" % os.path.splitext(os.path.basename(bam_file))[0])
-                if os.path.exists("%s.zip" % fastqc_outdir):
-                    os.remove("%s.zip" % fastqc_outdir)
-                if not os.path.exists(sentry_file):
+                tx_fastqc_out = os.path.join(tx_tmp_dir, "%s_fastqc" % fastqc_name)
+                tx_combo_file = os.path.join(tx_tmp_dir, "%s_fastqc.html" % fastqc_name)
+                if os.path.exists("%s.zip" % tx_fastqc_out):
+                    os.remove("%s.zip" % tx_fastqc_out)
+                if not os.path.exists(sentry_file) and os.path.exists(tx_combo_file):
+                    utils.safe_makedir(fastqc_out)
+                    shutil.copy(os.path.join(tx_fastqc_out, "fastqc_data.txt"), fastqc_out)
+                    shutil.move(tx_combo_file, sentry_file)
+                elif not os.path.exists(sentry_file):
                     if os.path.exists(fastqc_out):
                         shutil.rmtree(fastqc_out)
-                    shutil.move(fastqc_outdir, fastqc_out)
+                    shutil.move(tx_fastqc_out, fastqc_out)
         if ds_bam and os.path.exists(ds_bam):
             os.remove(ds_bam)
     parser = FastQCParser(fastqc_out)
@@ -456,7 +529,8 @@ def _run_gemini_stats(bam_file, data, out_dir):
     """Retrieve high level variant statistics from Gemini.
     """
     out = {}
-    gemini_db = data.get("variants", [{}])[0].get("population", {}).get("db")
+    gemini_db = (data.get("variants", [{}])[0].get("population", {}).get("db") 
+                 if data.get("variants") else None)
     if gemini_db:
         gemini_stat_file = "%s-stats.yaml" % os.path.splitext(gemini_db)[0]
         if not utils.file_uptodate(gemini_stat_file, gemini_db):
