@@ -3,12 +3,13 @@
 This handles two methods of getting processing information: from a Galaxy
 next gen LIMS system or an on-file YAML configuration.
 """
+from contextlib import closing
 import copy
 import itertools
 import os
-from contextlib import closing
-
 import string
+
+import toolz as tz
 import yaml
 
 from bcbio import utils
@@ -17,7 +18,11 @@ from bcbio.illumina import flowcell
 from bcbio.pipeline import alignment, config_utils, genome
 from bcbio.variation import effects, genotype, population
 from bcbio.variation.cortex import get_sample_name
-from bcbio.bam.fastq import open_fastq
+from bcbio.bam.fastq import open_fastq, is_fastq
+
+
+ALGORITHM_NOPATH_KEYS = ["variantcaller", "realign", "recalibrate",
+                         "phasing", "svcaller", "jointcaller", "tools_off"]
 
 def organize(dirs, config, run_info_yaml):
     """Organize run information from a passed YAML file or the Galaxy API.
@@ -123,7 +128,7 @@ def _check_for_batch_clashes(xs):
     names = set([x["description"] for x in xs])
     dups = set([])
     for x in xs:
-        batches = utils.get_in(x, ("metadata", "batch"))
+        batches = tz.get_in(("metadata", "batch"), x)
         if batches:
             if not isinstance(batches, (list, tuple)):
                 batches = [batches]
@@ -150,20 +155,20 @@ def _check_for_misplaced(xs, subkey, other_keys):
                                    ["% 15s | % 15s | % 15s" % (a, b, c) for (a, b, c) in problems]))
 
 ALGORITHM_KEYS = set(["platform", "aligner", "bam_clean", "bam_sort",
-                      "trim_reads", "adapters", "custom_trim",
+                      "trim_reads", "adapters", "custom_trim", "kraken",
                       "align_split_size", "quality_bin",
                       "quality_format", "write_summary",
                       "merge_bamprep", "coverage",
                       "coverage_interval", "ploidy",
-                      "variantcaller", "variant_regions",
-                      "mark_duplicates", "svcaller", "recalibrate",
+                      "variantcaller", "jointcaller", "variant_regions", "effects",
+                      "mark_duplicates", "svcaller", "svvalidate", "recalibrate",
                       "realign", "phasing", "validate",
                       "validate_regions", "validate_genome_build",
                       "clinical_reporting", "nomap_split_size",
                       "nomap_split_targets", "ensemble", "background",
                       "disambiguate", "strandedness", "fusion_mode", "min_read_length",
                       "coverage_depth_min", "coverage_depth_max", "min_allele_fraction", "remove_lcr",
-                      "archive", "tools_off"] +
+                      "archive", "tools_off", "assemble_transcripts"] +
                      # back compatibility
                       ["coverage_depth"])
 
@@ -180,6 +185,15 @@ def _check_algorithm_keys(item):
                          "See configuration documentation for supported options:\n%s\n"
                          % (problem_keys, url))
 
+def _check_toplevel_misplaced(item):
+    """Check for algorithm keys accidentally placed at the top level.
+    """
+    problem_keys = [k for k in item.keys() if k in ALGORITHM_KEYS]
+    if len(problem_keys) > 0:
+        raise ValueError("Unexpected configuration keywords found in top level of %s: %s\n"
+                         "This should be placed in the 'algorithm' section."
+                         % (item["description"], problem_keys))
+
 
 def _detect_fastq_format(in_file, MAX_RECORDS=1000):
     ranges = {"sanger": (33, 73),
@@ -271,6 +285,7 @@ def _check_sample_config(items, in_file):
                          ["resources", "metadata", "analysis",
                           "description", "genome_build", "lane", "files"])
 
+    [_check_toplevel_misplaced(x) for x in items]
     [_check_algorithm_keys(x) for x in items]
     [_check_aligner(x) for x in items]
     [_check_variantcaller(x) for x in items]
@@ -281,13 +296,16 @@ def _check_sample_config(items, in_file):
 def _file_to_abs(x, dnames):
     """Make a file absolute using the supplied base directory choices.
     """
-    if os.path.isabs(x):
+    if x is None or os.path.isabs(x):
         return x
+    elif isinstance(x, basestring) and x.lower() == "none":
+        return None
     else:
         for dname in dnames:
-            normx = os.path.normpath(os.path.join(dname, x))
-            if os.path.exists(normx):
-                return normx
+            if dname:
+                normx = os.path.normpath(os.path.join(dname, x))
+                if os.path.exists(normx):
+                    return normx
         raise ValueError("Did not find input file %s in %s" % (x, dnames))
 
 def _normalize_files(item, fc_dir):
@@ -300,6 +318,7 @@ def _normalize_files(item, fc_dir):
             files = [files]
         fastq_dir = flowcell.get_fastq_dir(fc_dir) if fc_dir else os.getcwd()
         files = [_file_to_abs(x, [os.getcwd(), fc_dir, fastq_dir]) for x in files]
+        files = [x for x in files if x]
         _sanity_check_files(item, files)
         item["files"] = files
     return item
@@ -318,6 +337,8 @@ def _sanity_check_files(item, files):
     elif file_type == "fastq":
         if len(files) not in [1, 2]:
             msg = "Expect either 1 (single end) or 2 (paired end) fastq inputs"
+        if len(files) == 2 and files[0] == files[1]:
+            msg = "Expect both fastq files to not be the same"
     if msg:
         raise ValueError("%s for %s: %s" % (msg, item.get("description", ""), files))
 
@@ -350,7 +371,7 @@ def _run_info_from_yaml(fc_dir, run_info_yaml, config):
             item["lane"] = str(i + 1)
         item["lane"] = _clean_characters(str(item["lane"]))
         if "description" not in item:
-            if len(item.get("files", [])) == 1 and item["files"][0].endswith(".bam"):
+            if _item_is_bam(item):
                 item["description"] = get_sample_name(item["files"][0])
             else:
                 raise ValueError("No `description` sample name provided for input #%s" % (i + 1))
@@ -367,8 +388,7 @@ def _run_info_from_yaml(fc_dir, run_info_yaml, config):
             item["upload"] = upload
         item["algorithm"] = _replace_global_vars(item["algorithm"], global_vars)
         item["algorithm"] = genome.abs_file_paths(item["algorithm"],
-                                                  ignore_keys=["variantcaller", "realign", "recalibrate",
-                                                               "phasing", "svcaller"])
+                                                  ignore_keys=ALGORITHM_NOPATH_KEYS)
         item["algorithm"] = _add_algorithm_defaults(item["algorithm"])
         item["rgnames"] = prep_rg_names(item, config, fc_name, fc_date)
         item["test_run"] = global_config.get("test_run", False)
@@ -376,6 +396,10 @@ def _run_info_from_yaml(fc_dir, run_info_yaml, config):
     _check_sample_config(run_details, run_info_yaml)
     return run_details
 
+def _item_is_bam(item):
+    files = item.get("files", [])
+    return len(files) == 1 and files[0].endswith(".bam")
+
 def _add_algorithm_defaults(algorithm):
     """Central location specifying defaults for algorithm inputs.
 
