@@ -21,6 +21,7 @@ _cl_progs = [{"cmd": "bamtofastq", "name": "biobambam",
              {"cmd": "bedtools", "args": "--version", "stdout_flag": "bedtools"},
              {"cmd": "bowtie2", "args": "--version", "stdout_flag": "bowtie2-align version"},
              {"cmd": "bwa", "stdout_flag": "Version:"},
+             {"cmd": "chanjo" }, 
              {"cmd": "cufflinks", "stdout_flag": "cufflinks"},
              {"cmd": "cutadapt", "args": "--version"},
              {"cmd": "fastqc", "args": "--version", "stdout_flag": "FastQC"},
@@ -245,6 +246,15 @@ def write_versions(dirs, config=None, is_wrapper=False):
                 out_handle.write("{program},{version}\n".format(**p))
     return out_file
 
+def get_version_manifest(name):
+    """Retrieve a version from the currently installed manifest.
+    """
+    manifest_vs = _get_versions_manifest()
+    for x in manifest_vs:
+        if x["program"] == name:
+            return x.get("version", "")
+    return ""
+
 def add_subparser(subparsers):
     """Add command line option for exporting version information.
     """
