@@ -3,9 +3,12 @@
 import datetime
 import os
 
+import toolz as tz
+
 from bcbio import log, utils
 from bcbio.upload import shared, filesystem, galaxy, s3
 from bcbio.pipeline import run_info
+import bcbio.pipeline.datadict as dd
 
 _approaches = {"filesystem": filesystem,
                "galaxy": galaxy,
@@ -37,9 +40,18 @@ def _get_files(sample):
         return _get_files_rnaseq(sample)
     elif analysis.lower() in ["chip-seq"]:
         return _get_files_chipseq(sample)
+    elif analysis.lower() in ["sailfish"]:
+        return _get_files_sailfish(sample)
     else:
         return []
 
+def _get_files_sailfish(sample):
+    out = []
+    out.append({"path": sample["sailfish_dir"],
+                "type": "directory",
+                "ext": "sailfish"})
+    return _add_meta(out, sample)
+
 def _get_files_rnaseq(sample):
     out = []
     algorithm = sample["config"]["algorithm"]
@@ -65,7 +77,8 @@ def _add_meta(xs, sample=None, config=None):
             if isinstance(sample["name"], (tuple, list)):
                 name = sample["name"][-1]
             else:
-                name = "%s-%s" % (sample["name"], run_info.clean_name(sample["description"]))
+                name = "%s-%s" % (sample["name"],
+                                  run_info.clean_name(sample["description"]))
             x["sample"] = name
         if config:
             if "fc_name" in config and "fc_date" in config:
@@ -84,8 +97,16 @@ def _get_files_variantcall(sample):
     out = _maybe_add_alignment(algorithm, sample, out)
     out = _maybe_add_variant_file(algorithm, sample, out)
     out = _maybe_add_sv(algorithm, sample, out)
+    out = _maybe_add_validate(algorithm, sample, out)
     return _add_meta(out, sample)
 
+def _maybe_add_validate(algorith, sample, out):
+    for i, plot in enumerate(tz.get_in(("validate", "grading_plots"), sample, [])):
+        out.append({"path": plot,
+                    "type": "pdf",
+                    "ext": "validate%s" % ("" if i == 0 else "-%s" % (i + 1))})
+    return out
+
 def _maybe_add_variant_file(algorithm, sample, out):
     if sample.get("align_bam") is not None and sample.get("vrn_file"):
         for x in sample["variants"]:
@@ -101,6 +122,17 @@ def _maybe_add_sv(algorithm, sample, out):
     if sample.get("align_bam") is not None and sample.get("sv"):
         for svcall in sample["sv"]:
             out.extend(_get_variant_file(svcall, ("vrn_file",)))
+            out.extend(_get_variant_file(svcall, ("bedpe_file",)))
+            if "sample_bed" in svcall:
+                out.append({"path": svcall["sample_bed"],
+                            "type": "bed",
+                            "ext": "%s-sample" % svcall["variantcaller"],
+                            "variantcaller": svcall["variantcaller"]})
+            if "validate" in svcall:
+                out.append({"path": svcall["validate"],
+                            "type": "csv",
+                            "ext": "%s-validate" % svcall["variantcaller"],
+                            "variantcaller": svcall["variantcaller"]})
     return out
 
 def _get_variant_file(x, key):
@@ -195,7 +227,8 @@ def _maybe_add_cufflinks(algorithm, sample, out):
 
 def _has_alignment_file(algorithm, sample):
     return (((algorithm.get("aligner") or algorithm.get("realign")
-              or algorithm.get("recalibrate")) and
+              or algorithm.get("recalibrate") or algorithm.get("bam_clean")
+              or algorithm.get("mark_duplicates")) and
               algorithm.get("merge_bamprep", True)) and
               sample.get("work_bam") is not None)
 
@@ -236,5 +269,13 @@ def _get_files_project(sample, upload_config):
         out.append({"path": sample["combined_counts"]})
     if "annotated_combined_counts" in sample:
         out.append({"path": sample["annotated_combined_counts"]})
+    if "combined_fpkm" in sample:
+        out.append({"path": sample["combined_fpkm"]})
+    if "combined_fpkm_isoform" in sample:
+        out.append({"path": sample["combined_fpkm_isoform"]})
+    if "assembled_gtf" in sample:
+        out.append({"path": sample["assembled_gtf"]})
+    if dd.get_dexseq_counts(sample):
+        out.append({"path": dd.get_dexseq_counts(sample)})
 
     return _add_meta(out, config=upload_config)
