@@ -10,6 +10,12 @@ import itertools
 import functools
 import random
 import ConfigParser
+import collections
+import fnmatch
+import subprocess
+
+import toolz as tz
+import yaml
 try:
     from concurrent import futures
 except ImportError:
@@ -17,10 +23,6 @@ except ImportError:
         import futures
     except ImportError:
         futures = None
-import collections
-import yaml
-import fnmatch
-
 
 @contextlib.contextmanager
 def cpmap(cores=1):
@@ -47,7 +49,6 @@ def map_wrap(f):
         return apply(f, *args, **kwargs)
     return wrapper
 
-
 def transform_to(ext):
     """
     Decorator to create an output filename from an output filename with
@@ -271,6 +272,17 @@ def remove_safe(f):
     except OSError:
         pass
 
+def file_plus_index(fname):
+    """Convert a file name into the file plus required indexes.
+    """
+    exts = {".vcf": ".idx", ".bam": ".bai", ".vcf.gz": ".tbi", ".bed.gz": ".tbi",
+            ".fq.gz": ".gbi"}
+    ext = splitext_plus(fname)[-1]
+    if ext in exts:
+        return [fname, fname + exts[ext]]
+    else:
+        return [fname]
+
 def symlink_plus(orig, new):
     """Create relative symlinks and handle associated biological index files.
     """
@@ -378,14 +390,7 @@ def get_in(d, t, default=None):
     example: get_in({1: {2: 3}}, (1, 2)) -> 3
     example: get_in({1: {2: 3}}, (2, 3)) -> {}
     """
-    result = reduce(lambda d, t: d.get(t, {}), t, d)
-    if result is False:
-        return result
-    elif not result:
-        return default
-    else:
-        return result
-
+    return tz.get_in(t, d, default)
 
 def flatten(l):
     """
@@ -538,3 +543,21 @@ def dictapply(d, fn):
         else:
             d[k] = fn(v)
     return d
+
+def R_package_path(package):
+    """
+    return the path to an installed R package
+    """
+    cmd = "Rscript -e 'find.package(\"{package}\")'"
+    try:
+        output = subprocess.check_output(cmd.format(**locals()), shell=True)
+    except subprocess.CalledProcessError, e:
+        return None
+    for line in output.split("\n"):
+        if "[1]" not in line:
+            continue
+        dirname = line.split("[1]")[1].replace("\"", "").strip()
+        if os.path.exists(dirname):
+            return dirname
+    return None
+
