@@ -1,11 +1,10 @@
 """Bayesian variant calling with FreeBayes.
 
-http://bioinformatics.bc.edu/marthlab/FreeBayes
+https://github.com/ekg/freebayes
 """
 
 from collections import namedtuple
 import os
-import shutil
 
 try:
     import vcf
@@ -17,8 +16,9 @@ from bcbio.distributed.transaction import file_transaction
 from bcbio.pipeline import config_utils
 from bcbio.pipeline.shared import subset_variant_regions
 from bcbio.provenance import do
-from bcbio.variation import annotation, ploidy
-from bcbio.variation.vcfutils import get_paired_bams, is_paired_analysis, bgzip_and_index
+from bcbio.variation import annotation, bedutils, ploidy
+from bcbio.variation.vcfutils import (get_paired_bams, is_paired_analysis,
+                                      bgzip_and_index, move_vcf)
 
 def region_to_freebayes(region):
     if isinstance(region, (list, tuple)):
@@ -28,10 +28,16 @@ def region_to_freebayes(region):
         return region
 
 def _freebayes_options_from_config(items, config, out_file, region=None):
+    """Prepare standard options from configuration input.
+
+    Input BED target files are merged to avoid overlapping regions which
+    cause FreeBayes to call multiple times.
+    """
     opts = []
     opts += ["--ploidy", str(ploidy.get_ploidy(items, region))]
 
-    variant_regions = utils.get_in(config, ("algorithm", "variant_regions"))
+    variant_regions = bedutils.merge_overlaps(utils.get_in(config, ("algorithm", "variant_regions")),
+                                              items[0])
     target = subset_variant_regions(variant_regions, region, out_file, items)
     if target:
         if isinstance(target, basestring) and os.path.isfile(target):
@@ -43,9 +49,9 @@ def _freebayes_options_from_config(items, config, out_file, region=None):
         opts += resources["options"]
     if "--min-alternate-fraction" not in " ".join(opts) and "-F" not in " ".join(opts):
         # add minimum reportable allele frequency, for which FreeBayes defaults to 20
-         min_af = float(utils.get_in(config, ("algorithm",
-                                              "min_allele_fraction"),20)) / 100.0
-         opts += ["--min-alternate-fraction", str(min_af)]
+        min_af = float(utils.get_in(config, ("algorithm",
+                                             "min_allele_fraction"), 20)) / 100.0
+        opts += ["--min-alternate-fraction", str(min_af)]
     return opts
 
 def run_freebayes(align_bams, items, ref_file, assoc_files, region=None,
@@ -108,10 +114,14 @@ def _run_freebayes_paired(align_bams, items, ref_file, assoc_files,
         with file_transaction(out_file) as tx_out_file:
             paired = get_paired_bams(align_bams, items)
             if not paired.normal_bam:
-                raise ValueError("Require both tumor and normal BAM files for FreeBayes cancer calling")
+                return _run_freebayes_caller(align_bams, items, ref_file,
+                                             assoc_files, region, out_file)
+                #raise ValueError("Require both tumor and normal BAM files for FreeBayes cancer calling")
 
             vcfsamplediff = config_utils.get_program("vcfsamplediff", config)
             vcffilter = config_utils.get_program("vcffilter", config)
+            vcfallelicprimitives = config_utils.get_program("vcfallelicprimitives", config)
+            vcfstreamsort = config_utils.get_program("vcfstreamsort", config)
             freebayes = config_utils.get_program("freebayes", config)
             opts = " ".join(_freebayes_options_from_config(items, config, out_file, region))
             opts += " -f {}".format(ref_file)
@@ -120,7 +130,7 @@ def _run_freebayes_paired(align_bams, items, ref_file, assoc_files,
                 # FreeBayes defaults to 20%, but use 10% by default for the
                 # tumor case
                 min_af = float(utils.get_in(paired.tumor_config, ("algorithm",
-                                                                  "min_allele_fraction"),10)) / 100.0
+                                                                  "min_allele_fraction"), 10)) / 100.0
                 opts += " --min-alternate-fraction %s" % min_af
             # NOTE: The first sample name in the vcfsamplediff call is
             # the one supposed to be the *germline* one
@@ -133,7 +143,8 @@ def _run_freebayes_paired(align_bams, items, ref_file, assoc_files,
                   "{opts} {paired.tumor_bam} {paired.normal_bam} "
                   "| {vcffilter} -f 'QUAL > 1' -s "
                   "| {vcfsamplediff} VT {paired.normal_name} {paired.tumor_name} - "
-                  "{compress_cmd} >  {tx_out_file}")
+                  "| {vcfallelicprimitives} | {vcfstreamsort} "
+                  "{compress_cmd} > {tx_out_file}")
             bam.index(paired.tumor_bam, config)
             bam.index(paired.normal_bam, config)
             do.run(cl.format(**locals()), "Genotyping paired variants with FreeBayes", {})
@@ -144,14 +155,6 @@ def _run_freebayes_paired(align_bams, items, ref_file, assoc_files,
     return ann_file
 
 
-def _move_vcf(orig_file, new_file):
-    """Move a VCF file with associated index.
-    """
-    for ext in ["", ".idx", ".tbi"]:
-        to_move = orig_file + ext
-        if os.path.exists(to_move):
-            shutil.move(to_move, new_file + ext)
-
 def _clean_freebayes_output(line):
     """Clean FreeBayes output to make post-processing with GATK happy.
 
@@ -189,8 +192,8 @@ def clean_vcf_output(orig_file, clean_fn, name="clean"):
                         update_line = clean_fn(line)
                         if update_line:
                             out_handle.write(update_line)
-        _move_vcf(orig_file, "{0}.orig".format(orig_file))
-        _move_vcf(out_file, orig_file)
+        move_vcf(orig_file, "{0}.orig".format(orig_file))
+        move_vcf(out_file, orig_file)
         with open(out_file, "w") as out_handle:
             out_handle.write("Moved to {0}".format(orig_file))
 
@@ -204,7 +207,8 @@ def fix_somatic_calls(in_file, config):
     # HACK: Needed to replicate the structure used by PyVCF
     Info = namedtuple('Info', ['id', 'num', 'type', 'desc'])
     somatic_info = Info(id='SOMATIC', num=0, type='Flag', desc='Somatic event')
-
+    Filter = namedtuple('Filter', ['id', 'desc'])
+    reject_filter = Filter(id='REJECT', desc='Rejected as non-SOMATIC or by quality')
     # NOTE: PyVCF will write an uncompressed VCF
     base, ext = utils.splitext_plus(in_file)
     name = "somaticfix"
@@ -214,23 +218,26 @@ def fix_somatic_calls(in_file, config):
         reader = vcf.VCFReader(filename=in_file)
         # Add info to the header of the reader
         reader.infos["SOMATIC"] = somatic_info
-
+        reader.filters["REJECT"] = reject_filter
         with file_transaction(out_file) as tx_out_file:
             with open(tx_out_file, "wb") as handle:
                 writer = vcf.VCFWriter(handle, template=reader)
                 for record in reader:
                     # Handle FreeBayes
+                    is_somatic = False
                     if "VT" in record.INFO:
                         if record.INFO["VT"] == "somatic":
                             record.add_info("SOMATIC", True)
+                            is_somatic = True
                         # Discard old record
                         del record.INFO["VT"]
-
+                    if not is_somatic:
+                        record.add_filter("REJECT")
                     writer.write_record(record)
 
         # Re-compress the file
         out_file = bgzip_and_index(out_file, config)
-        _move_vcf(in_file, "{0}.orig".format(in_file))
-        _move_vcf(out_file, in_file)
+        move_vcf(in_file, "{0}.orig".format(in_file))
+        move_vcf(out_file, in_file)
         with open(out_file, "w") as out_handle:
             out_handle.write("Moved to {0}".format(in_file))
