@@ -2,13 +2,16 @@
 
 from distutils.version import LooseVersion
 import os
+import re
+import shutil
 
-from bcbio import bam, broad
-from bcbio.utils import file_exists, get_in
+from bcbio import bam, broad, utils
+from bcbio.utils import file_exists, get_in, open_gzipsafe, remove_safe
 from bcbio.distributed.transaction import file_transaction
 from bcbio.variation.realign import has_aligned_reads
 from bcbio.pipeline.shared import subset_variant_regions
-from bcbio.variation import bamprep, vcfutils
+from bcbio.variation import bamprep, vcfutils, scalpel
+from bcbio.variation.vcfutils import bgzip_and_index
 from bcbio.log import logger
 
 _PASS_EXCEPTIONS = set(["java.lang.RuntimeException: "
@@ -70,8 +73,10 @@ def _mutect_call_prep(align_bams, items, ref_file, assoc_files,
 
     paired = vcfutils.get_paired_bams(align_bams, items)
     params = ["-R", ref_file, "-T", "MuTect", "-U", "ALLOW_N_CIGAR_READS"]
-    params += ["--downsample_to_coverage", max(200, get_in(paired.tumor_config,
-                                                           ("algorithm", "coverage_depth_max"), 10000))]
+    # if coverage_depth_max is not given, default to 10000
+    downsample_cov = get_in(paired.tumor_config, ("algorithm", "coverage_depth_max"), 10000)
+    # if coverage_depth_max is zero, default to Broad default value (currently 1500)
+    params += ["--downsample_to_coverage", max(1500, downsample_cov)] if downsample_cov > 0 else []
     params += ["--read_filter", "NotPrimaryAlignment"]
     params += ["-I:tumor", paired.tumor_bam]
     params += ["--tumor_sample_name", paired.tumor_name]
@@ -87,16 +92,14 @@ def mutect_caller(align_bams, items, ref_file, assoc_files, region=None,
                   out_file=None):
     """Run the MuTect paired analysis algorithm.
     """
+    config = items[0]["config"]
     if out_file is None:
         out_file = "%s-paired-variants.vcf.gz" % os.path.splitext(align_bams[0])[0]
     if not file_exists(out_file):
         base_config = items[0]["config"]
         broad_runner = broad.runner_from_config(base_config, "mutect")
-        if "appistry" in broad_runner.get_mutect_version():
-            out_file_mutect = (out_file.replace(".vcf", "-mutect.vcf")
-                               if "vcf" in out_file else out_file + "-mutect.vcf")
-        else:
-            out_file_mutect = out_file
+        out_file_mutect = (out_file.replace(".vcf", "-mutect.vcf")
+                           if "vcf" in out_file else out_file + "-mutect.vcf")
         broad_runner, params = \
             _mutect_call_prep(align_bams, items, ref_file, assoc_files,
                                    region, out_file_mutect)
@@ -108,7 +111,29 @@ def mutect_caller(align_bams, items, ref_file, assoc_files, region=None,
             # Rationale: MuTect writes another table to stdout, which we don't need
             params += ["--vcf", tx_out_file, "-o", os.devnull]
             broad_runner.run_mutect(params)
-        if "appistry" in broad_runner.get_mutect_version():
+        _rename_allelic_fraction_field(out_file_mutect,config)
+        disable_SID = True # SID isn't great, so use Scalpel instead
+        if "appistry" not in broad_runner.get_mutect_version() or disable_SID:
+            # Scalpel InDels
+            is_paired = "-I:normal" in params
+            out_file_indels = (out_file.replace(".vcf", "-somaticIndels.vcf")
+                               if "vcf" in out_file else out_file + "-somaticIndels.vcf")
+            if scalpel.is_installed(items[0]["config"]):
+                with file_transaction(out_file_indels) as tx_out_file2:
+                    if not is_paired:
+                        scalpel._run_scalpel_caller(align_bams, items, ref_file, assoc_files,
+                                                    region=region, out_file=tx_out_file2)
+                    else:
+                        scalpel._run_scalpel_paired(align_bams, items, ref_file, assoc_files,
+                                                    region=region, out_file=tx_out_file2)
+                out_file = vcfutils.combine_variant_files(orig_files=[out_file_mutect, out_file_indels],
+                                                          out_file=out_file,
+                                                          ref_file=items[0]["sam_ref"],
+                                                          config=items[0]["config"],
+                                                          region=region)
+            else:
+                utils.symlink_plus(out_file_mutect, out_file)
+        else:
             # SomaticIndelDetector modifications
             out_file_indels = (out_file.replace(".vcf", "-somaticIndels.vcf")
                                if "vcf" in out_file else out_file + "-somaticIndels.vcf")
@@ -156,3 +181,28 @@ def _SID_call_prep(align_bams, items, ref_file, assoc_files, region=None, out_fi
         params += ["-L", bamprep.region_to_gatk(region), "--interval_set_rule",
                    "INTERSECTION"]
     return params
+
+
+def _rename_allelic_fraction_field(orig_file,config):
+    """Rename allelic fraction field in mutect output
+       from FA to FREQ to standarize with other tools
+    """
+    tmp_file = orig_file.replace(".vcf.gz","-fix.vcf")
+    with file_transaction(tmp_file) as tx_in_file:       
+        with open(tx_in_file,'w') as out_handle:
+            with open_gzipsafe(orig_file) as handle:
+                for line in handle:
+                    if line.startswith("##FORMAT=<ID=FA"):
+                        line = line.replace("=FA","=FREQ")
+                    if not line.startswith("#"):
+                        line = line.replace("FA","FREQ")
+                    out_handle.write(line)
+    out_file = orig_file.replace(".gz","")
+    remove_safe(orig_file)
+    shutil.move(tmp_file, out_file )
+    with open(tmp_file, "w") as out_handle:
+        out_handle.write("Moved to {0}".format(out_file))
+    out_file = bgzip_and_index(out_file, config)
+
+
+
