@@ -86,7 +86,7 @@ def _create_validate_config_file(vrn_file, rm_file, rm_interval_file, rm_genome,
     with open(config_file, "w") as out_handle:
         out = _create_validate_config(vrn_file, rm_file, rm_interval_file, rm_genome,
                                       base_dir, data)
-        yaml.dump(out, out_handle, default_flow_style=False, allow_unicode=False)
+        yaml.safe_dump(out, out_handle, default_flow_style=False, allow_unicode=False)
     return config_file
 
 def _create_validate_config(vrn_file, rm_file, rm_interval_file, rm_genome,
